#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 12/30/2001
#################################################################################
#
# Text Configuration - Script part of Webchat designed to modify documents on database
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################
use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Audit table
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control

#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %config;							# System hash

#################################################################################
# Program Area
#################################################################################

	# Initial handle
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationAdmin())
	{
		# Title
		print $cgi->h1('Document Administration'), "\n";

		# If no DOCTYPE then list options 
		if (!defined($param->{'DOCTYPE'}))
		{ DocumentSelect($database, $cgi, $param); }			
	
		# If only DOCTYPE is specified then allow to modify
		elsif ((defined($param->{'DOCTYPE'})) && (!defined($param->{'DOCDATA'})))
		{
			DocumentSelect($database, $cgi, $param);
			DocumentModify($database, $cgi, $param); 		
		}

		# If only DOCTYPE and DOCDATA are specified
		elsif ((defined($param->{'DOCTYPE'})) && (defined($param->{'DOCDATA'})))
		{
			DocumentSelect($database, $cgi, $param);
			DocumentModify($database, $cgi, $param); 		
		}
	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Document Modify
#
# Simply modifies documents available and allows one to select them.

sub DocumentModify						# Database, CGI
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list

	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $audit;						# Audit handle
	my $document;						# Document
	

	#####################
	# Program area	
	
	# Audit handle
	$audit = new Ethereal::Audit($database, $cgi, $param);

	# If defined save
	if (defined($param->{'DOCDATA'}))
	{
		$data->do("UPDATE WebchatText 
			SET WebchatTextContent=? 
			WHERE WebchatTextName=?", {},  

			$param->{'DOCDATA'}, 
			$param->{'DOCTYPE'});

		# Audit Trail
		$audit->Document($param->{'DOCTYPE'});

	}

	# Retrieve contents
	($document) = $database->DataGet("SELECT WebchatTextContent
		FROM WebchatText
		WHERE WebchatTextName=?", $param->{'DOCTYPE'});

	# Cleanup
	$param->Cleanup('DOCDATA');


	# Start from
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";	

	# Display HTML and information
	print "\t<B>Opened document: $param->{DOCTYPE}</B>\n\t<BR>\n";
	print "\t", $cgi->textarea(-name=>'DOCDATA',-rows=>25,-columns=>85,-wrap=>'OFF',-default=>$document), "\n";


	# End of form
	print "\t", $cgi->p($cgi->submit('Save Changes')), "\n";
	print $cgi->end_form(), "\n";
}


#####################
# DocumentSelect
#
# Simply lists documents available and allows one to select them.

sub DocumentSelect						# Database, CGI
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list

	my $list;						# Default list unit
	my @list;						# List of fields


	#####################
	# Program area

	# Retrieve list of documents
	$database->GetListDocuments(\@list);

	# Determin defaults
	$list = (defined($param->{'DOCTYPE'})) ? $param->{'DOCTYPE'} : $list[0];	


        # Form data		
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}), "\n";

	# Prompt and Display Document choices
	print "\t", $cgi->p('Document list: ', $cgi->popup_menu('DOCTYPE', \@list, $list)), "\n";
	print "\t", $cgi->p($cgi->submit('Open Document')), "\n";


	print $cgi->end_form(), "\n";				# Ending HTML form
}
