#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 08/30/2001
#################################################################################
#
# System Who - Script part of Webchat designed to list who is currently on all realms
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Who;						# Who is on


#################################################################################
# Data Members
#################################################################################
my $database;							# Database handle
my $cgi;							# Common gateway interface handle
my $login;							# Login handle
my $param;							# Parameter handle
my $who;							# Who handle

my $url;							# URL
my $inline;							# Inline parameters

my %realm;							# Realm hash


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);	

	# Fetch hash
	$database->GetHashRealm(\%realm);

	# Fetch URL
	$url = $cgi->url();

	# Retreive parameters
	$inline = $param->EmbedInline($cgi, %{$param});

	# Document header
	print $cgi->header(-type=>'text/html',
		-refresh=>"300;URL=$url$inline");
	$database->DocumentGetHeader();


	# Login and Who handle
	$login = new Ethereal::Login($database, $cgi, $param);
	$who   = new Ethereal::Who($database, $cgi, $param);	

        # Authenthication
        if ($login->GetVerificationAdmin())
        {
		# Display title
		print "$realm{'TagTitleWho'}\n<BR>\n";

		# Display who is on
		$who->WhoSystem(\%realm);
	}

	# Document footer
	$database->DocumentGetFooter();
