#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 09/05/2001
#################################################################################
#
# Portal - Script part of Webchat designed to authenthicate and and lead to realms
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Weather;						# Weather handler
use Ethereal::Who;						# Who is on


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash

my %portal;							# Portal hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Link with hash
	$database->GetHashPortal(\%portal);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# If undefined list realms, puppets and links
		unless (defined($param->{'THIRDCHAPTER'}))
		{ 
			SecondChapter($database, $cgi, $param, \%portal); 
		}

		# Otherwise open gateway
		elsif (defined($param->{'THIRDCHAPTER'}))
		{ 
			ThirdChapter($database, $cgi, $param, \%portal); 
		}
	}

	# Document footer
	$database->DocumentGetFooter();

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Second Chapter
#
# Unusual name given to a part of the script that will be exected once the user
# has been authenthicated.   

sub SecondChapter						# Database, CGI, Parameter list
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $portal   = shift;					# Portal hash

	my $table    = new Ethereal::Table($cgi);		# Table handler 

	my $inline;						# Inline embeding of data
	my $javascript;						# Javascript control
	my $num;						# Number
	my $realms;						# Formatted list of realms
	my $puppets;						# Formatted list of puppets
	my $url;						# Self referencing link

	my @realms;						# List of realms
	my @puppets;						# List of puppets



	#####################
	# Program area

	# Retreive URL
	$url = $cgi->url();


	# Javascript Options
	# Retreive Javascript choice
	($javascript) = $database->DataGetJavascript($param->{'USER'});
	
	# Possibly display Javascript
	if ($javascript eq 'yes')
	{ $database->DocumentGetJavascript(); }


	# Retreive and format values for Realms and Puppets
	$database->GetListPublicRealms(\@realms);	
	$database->GetListPuppetYours(\@puppets, $param->{'USER'});	

	$realms  = $cgi->scrolling_list('ROOM', \@realms, $realms[0], 10);
	$puppets = $cgi->scrolling_list('CHAR', \@puppets, $puppets[0], 5);

	# Retreive puppet numbers
	($num) = $database->DataGet('SELECT COUNT(*) 
		FROM  Puppet 
		WHERE PuppeteerLogin=?', $param->{'USER'});


	# Display of information
	# Table top
	print "<TABLE WIDTH=\"100%\">\n";

	# Row top
	print "\t<TR>\n\t\t<TD VALIGN=\"TOP\">\n";

	# Only display if puppets exist
	if ($num > 0)
	{
		# Realm Section
		# Start form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'THIRDCHAPTER', 'true'), "\n";

		# Table top
		$table->MakeTop(400);
		$table->MakeSingle($portal->{'TagSecRealm'});
		$table->MakeBlank();
	
		$table->MakeValid($portal->{'TagSelectPuppet'},  $puppets);
		$table->MakeBlank();
		$table->MakeValid($portal->{'TagSelectRealm'},   $realms);
		$table->MakeBlank();
		$table->MakeValid($portal->{'TagSelectPrivate'}, $cgi->textfield('ALTROOM', '', 20, 30));
		$table->MakeBlank();
	
		$table->MakeValid($cgi->submit($portal->{'TxtSubmitRealms'}), $cgi->reset($portal->{'TxtResetRealms'}));
		$table->MakeBottom();

		# End form
		print $cgi->end_form(), "\n";


		# Spacer
		print "<BR>\n";
	}

		
	# Row bottom
	print "\t\t</TD>\n";

	# Links
	# Determine priviliges
	my ($sup, $adm) = $database->DataGetPrivs($param->{'USER'});

	# Create inline link
	$inline = $param->EmbedInline($cgi, %{$param});


	# Row top
	print "\t\t<TD VALIGN=\"TOP\">\n";


	# Puppeteer Section
	print "$portal->{TagSecPuppeteer}\n";

	# Link handling
	if ($javascript eq 'no')
	{
		# Non javascript links
		print $cgi->ul(
		 $cgi->li("<A HREF=\"$portal->{LnkPuppeteer}$inline\" TARGET=\"_new\">$portal->{TagLnkPuppeteer}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkPuppet}$inline\" TARGET=\"_new\">$portal->{TagLnkPuppet}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkMail}$inline&LIST=True\" TARGET=\"_new\">$portal->{TagLnkMail}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkHome}$inline\" TARGET=\"_new\">$portal->{TagLnkHome}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkIgnore}$inline\" TARGET=\"_new\">$portal->{TagLnkIgnore}</A>"), "\n");
	}
	else
	{
		# Javascript links
		print $cgi->ul(
		 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{LnkPuppeteer}$inline',1)\">$portal->{TagLnkPuppeteer}</A>"),
		 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{LnkPuppet}$inline',1)\">$portal->{TagLnkPuppet}</A>"),
		 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{LnkMail}$inline&LIST=True',1)\">$portal->{TagLnkMail}</A>"),
		 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{LnkHome}$inline',1)\">$portal->{TagLnkHome}</A>"),
		 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{LnkIgnore}$inline',1)\">$portal->{TagLnkIgnore}</A>"), "\n");
	}

	# Spacer
	print "<BR>\n";


	# Administrative area
	if ($adm eq 'yes')
	{
		# Title
		print "$portal->{TagSecAdmin}\n";

		# Links
		print $cgi->ul(
		 $cgi->li("<A HREF=\"$portal->{LnkAdmDoc}$inline\" TARGET=\"_new\">$portal->{TagLnkAdmDoc}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkAdmChat}$inline\" TARGET=\"_new\">$portal->{TagLnkAdmChat}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkAdmRealm}$inline\" TARGET=\"_new\">$portal->{TagLnkAdmRealm}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkAdmAccess}$inline\" TARGET=\"_new\">$portal->{TagLnkAdmAccess}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkAdmUser}$inline\" TARGET=\"_new\">$portal->{TagLnkAdmUser}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkAdmWho}$inline\" TARGET=\"_new\">$portal->{TagLnkAdmWho}</A>"), "\n");

		# Spacer
		print "<BR>\n";
	}


	# Supervisory Area
	if ($sup eq 'yes')
	{
		# Title
		print "$portal->{TagSecSuper}\n";

		# Display following button if supervisor
		# No javascript
		if ($javascript eq 'no')
		{
			print $cgi->ul(
			 $cgi->li("<A HREF=\"$portal->{LnkRealm}$inline\" TARGET=\"_TOP\">$portal->{TagLnkSupervisor}</A>"), "\n"); 
		}
		# Javascript
		else
		{
			print $cgi->ul(
			 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{'LnkRealm'}$inline',1)\">$portal->{'TagLnkSupervisor'}</A>"), "\n"); 
		}

		# Spacer
		print "<BR>\n";
	}


	# Miscellaneous area
	print "$portal->{TagSecMisc}\n";


	# Link handling
	if ($javascript eq 'no')
	{
		# Non javascript links
		print $cgi->ul(
		 $cgi->li("<A HREF=\"$portal->{LnkCookie}$inline\" TARGET=\"_new\">$portal->{TagLnkCookie}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkBoard}$inline\" TARGET=\"_new\">$portal->{TagLnkBoard}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkGlobalWho}$inline\" TARGET=\"_blank\">$portal->{TagLnkGlobalWho}</A>"), "\n");
	}
	else
	{
		# Javascript links
		print $cgi->ul(
		 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{LnkCookie}$inline',1)\">$portal->{TagLnkCookie}</A>"),
		 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{LnkBoard}$inline',1)\">$portal->{TagLnkBoard}</A>"),
		 $cgi->li("<A HREF=\"javascript:popup_window('$portal->{LnkGlobalWho}$inline',1)\">$portal->{TagLnkGlobalWho}</A>"));
	}

	# End of table
	print "\t\t</TD>\n\t</TR>\n</TABLE>\n";
}


#####################
# Third Chapter
#
# Unusual name given to a part of the script that will be exected once the user  
# in his infinit wisdom has selected a puppet and realm

sub ThirdChapter						# Database, CGI, Parameter list
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $portal   = shift;					# Portal hash

	my $data     = $database->{'HANDLE'};			# Database connection handler

	my $who;						# Who is on handle

	my $choice;						# To be or not to be
	my $room;						# Room name
	my $info;						# Room information


	#####################
	# Program area

	# Cleanup
	delete $param->{'THIRDCHAPTER'};

	# Ensure puppet
	unless (defined($param->{'CHAR'}))
	{
		# Retreive default puppet
		($param->{'CHAR'}) = $database->DataGetDefault($param->{'USER'});
	}



	# Determine need to create a room
	if (defined($param->{'ALTROOM'}) && (length(($param->{'ALTROOM'})) > 2))
	{
		# Data Assignment
		$room = $param->{'ALTROOM'};

		# Retreive query information
		($choice) = $database->DataGetRealmExistence($room);

		# Create if not found
		unless (defined($choice))
		{
			$data->do("INSERT INTO Realm 
				(RealmName,
				 RealmLanguage,
				 RealmAccess,
				 RealmGuests,
				 RealmPublic,
				 RealmTags,
				 RealmImageHeight,
				 RealmImageWidth,
				 RealmTimestamp)
				VALUES(?, 'relaxed', 'relaxed', 'yes', 'no', 'relaxed', '0', '0', ?)", {}, 
			 $room, 
			 time); 			
		}
		
		# Assign one to the other (as to be called properly)
		$param->{'ROOM'} = $room;
	}

	# Retreive realm information
	($info) = $database->DataGetRealmDescription($param->{'ROOM'});

	
	# who is on handles
	$who = new Ethereal::Who($database, $cgi, $param);
	
	# Wrap with tables
	if (defined($info))
	{	
		# Wrap with tables
		print "<TABLE WIDTH=\"100%\">\n\t<TR>\n\t\t<TD VALIGN=\"top\">";

		# Retreive realm information
		$who->WhoEntrance();

		# Weather forcast
		WeatherHandler($database, $cgi, $param, $portal);

		# Mid section and info
		print "\n\t\t</TD>\n\t\t<TD VALIGN=\"top\">";
		print "$info\n";

		# Hind quarter
		print "\n\t\t</TD>\n\t</TR>\n</TABLE>";
	}

	# Simply display
	else
	{
		# Retreive realm information
		$who->WhoEntrance();

		# Weather forcast
		WeatherHandler($database, $cgi, $param, $portal);
	}
}


#####################
# Weather Handler
#
# The weather heandler is realm specific for which is it not listed on the
# module directly.   This will essentially show off the specific weather 
# patterns for the realm if found

sub WeatherHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $portal   = shift;					# Portal handle

	my $table    = new Ethereal::Table($cgi);		# Table handle

	my $for;						# Forcast handle
	my $metar;						# Coded weather forcast

	my $cond;						# Weather conditions
	my $sky;						# Sky conditions

	my %cond;						# Specific conditions


	#####################
	# Program area

	# Retreive forcast
	($metar) = $database->DataGetWeather($param->{'ROOM'});
	

	# Only complete if forcast
	if (defined($metar))
	{
		# Initialize handle
		$for = new Ethereal::Weather();

		# Parse
		$for->Parse($metar, \%cond);


		# Forcast handling
		$sky  = lc(join(", ", @{$cond{SKY}}));
		$cond = lc(join(", ", @{$cond{HUMAN}}));		

		# Conditions
		$cond = (length($cond) > 2) ? ucfirst("$cond, $sky") : ucfirst("$sky");
		$cond =~ s/,([^,.]+?)$/ and $1/s;


		# Title
		print "<BR>\n$portal->{'TagForcastTitle'}\n<BR>\n";

		# Display area
		$table->MakeTop(300);
		$table->MakeSingle("$portal->{TagForcastCond}<BR>$cond");

		# Wind
		if ($cond{WIND_MPH} != 0)
		{ $table->MakeSingle("$portal->{TagForcastWind}<BR>$cond{WIND_DIR_ENG} at $cond{WIND_KPH} KPH - $cond{WIND_MPH} MPH"); }

		# Temperature
		if ($cond{F_TEMP} != 0)
		{ $table->MakeSingle("$portal->{TagForcastTemp}<BR>$cond{C_TEMP}C - $cond{F_TEMP}F" ); }

		# Dew points
		if ($cond{F_DEW} != 0)
		{ $table->MakeSingle("$portal->{TagForcastDew} <BR>$cond{C_DEW}C - $cond{F_DEW}F"); }

		# Visibility
		if ($cond{VIS_MILE} != 0)
		{ $table->MakeSingle("$portal->{TagForcastVis} <BR>$cond{VIS_KILO} Km - $cond{VIS_MILE} Mi"); }

		# Barometer
		if ($cond{ALTI} != 0)
		{ $table->MakeSingle("$portal->{TagForcastBaro}<BR>$cond{ALTI} Inches - $cond{ALTK} Kilopascals"); }

		# Table bottom
		$table->MakeBottom();
	}
}
