#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 12/31/2001
#################################################################################
#
# Private - Script part of Webchat designed to send private messages
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Comm;						# Communication handler
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handling
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $comm;							# Communications panel
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my $count;							# Amount of targets
my $post;							# Temporary post
my $target;							# Individual targets
my @targets;							# Targets to send to

my %realm;							# Realm DBM


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Fetch target
	@targets = $cgi->param('TARGET');

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Fetch hash
	$database->GetHashRealm(\%realm);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login and Comm handles
	$login = new Ethereal::Login($database, $cgi, $param);
	$comm  = new Ethereal::Comm($database, $cgi);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Prevent bans
		$login->CheckLegality(\%realm);

		# Only do the following if posting is needed
		if (defined($param->{'PSEND'}))
		{
			# Only if target is valid
			if ($param->{'TARGET'} ne $realm{'TxtNone'})
			{
				# Retreive count
				$count = @targets;

				# Determin post
				$post = ($count > 1) ? "$realm{TagMulti} $param->{POST}" : "$param->{POST}";
			
				# Circle through hosts
				foreach $target (@targets)
				{
					# Set value of target
					$param->{'TARGET'} = $target;
					$param->{'POST'}   = $post;
		
					TargetHandler($database, $param, \%realm);
					$comm->DataWrite($param);
				}
			}
		}

		# Display panel
		DisplayHandler($database, $cgi, $param, \%realm);
	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

####################
# Display Handler
#
# Display prompt for a private message, this includes a list of all the puppets
# et cetera. 

sub DisplayHandler						# Database, Cgi, Param
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Cgi handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Realm options

	my $option   = new Ethereal::Option();			# Option handle
	my $table    = new Ethereal::Table($cgi);		# Table handle

	my $admin;						# Admin priviliges
	my $target;						# Target sent to
	my $who;						# List of puppets
	my $char;						# List of characters
	my $color;						# Colour choices
	my $face;						# Font face
	my $size;						# Text colour

	my $defcolor;						# Default colour
	my $defface;						# Default face
	my $defsize;						# Default size
	my $delay;						# Delay of time

	my @who;						# List of puppets
	my @char;						# List of characters
	my @color;						# Colour choices
	my @face;						# Font face
	my @size;						# Text colour



	####################
	# Program area

	# Calculate delay
	$delay = (time - $realm->{'SetTimeout'});

	# Populate arrays
	$option->Split(\@color, $realm->{'OptTextColour'});
	$option->Split(\@face,  $realm->{'OptTextFace'});
	$option->Split(\@size,  $realm->{'OptTextSize'});

	$database->GetListPuppetYours(\@char, $param->{'USER'});
	$database->GetListWho(\@who, $param->{'ROOM'}, $delay);

	
	# Administrative priviliges
	($admin) = $database->DataGetAdmin($param->{'USER'});

	# Retreive size and color
	($defsize, $defcolor, $defface) = $database->DataGetFont($param->{'CHAR'}, $color[0], $face[0]);

	# Allow for system messages
	if ($admin eq 'yes')
	{ push(@who, $realm->{'TxtSystem'}); }

	# Default selection
	unshift(@who, $realm->{'TxtNone'});


	# Targets to who
	$target = (defined($param->{'OTARGET'})) ? $param->{'OTARGET'} : $who[0];

	# Format buttons
	$who    = $cgi->scrolling_list('TARGET', \@who, $target, 5, 'true');
	$char	= $cgi->popup_menu('CHAR',  \@char,  $param->{'CHAR'});
	$color	= $cgi->popup_menu('COLOR', \@color, $defcolor);
	$face	= $cgi->popup_menu('FACE',  \@face,  $defface);
	$size   = $cgi->popup_menu('SIZE',  \@size,  $defsize);


	# HTML
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'PSEND', 'True'), "\n";

	# Table top
	$table->MakeTop();
	$table->MakeSingle($realm->{'SetTop'});

	# Prompt and space
	$table->MakeSingle($cgi->textarea(-name=>'POST',-rows=>3,-columns=>70,-wrap=>'VIRTUAL'));
	$table->MakeBlank();

	# Prompt for From, To, Colour, Puppet
	$table->MakeValid($realm->{'TagPuppet'},     $char);
	$table->MakeValid($realm->{'TagPuppetTo'},   $who);
	$table->MakeValid($realm->{'TagTextColour'}, $color);
	$table->MakeValid($realm->{'TagTextFace'},   $face);
	$table->MakeValid($realm->{'TagTextSize'},   $size);
	$table->MakeBlank();

	# Submit Button
	$table->MakeSingle($cgi->submit($realm->{'TxtSubmitButton'}));
	$table->MakeBottom();

	# Form bottom
	print $cgi->end_form();
}


#####################
# Target Handler
#
# Will determine target to send the message privately to.  This will also create a
# parameter called SOURCE which will receive a copy as well.

sub TargetHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Chat options

	my $puppeteer;						# Puppeteer to send to


	#####################
	# Program area

	# Only do changes if target exists
	if (defined($param->{'TARGET'}))
	{
		# Retreive puppeteer name
		$param->{'OTARGET'} = $param->{'TARGET'};	# Keep original value

		# Only retreive if needed
		if (($param->{'OTARGET'} ne 'dice roll') 
		 && ($param->{'OTARGET'} ne $realm->{'TxtSystem'}))
		{ 
			# Retreive and assign
			($puppeteer) = $database->DataGetPuppeteerLogin($param->{'TARGET'}); 
			$param->{'TARGET'} = $puppeteer;
		}

		# Assign values to parameters
		$param->{'SOURCE'} = $param->{'USER'};
	}
}
